@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CmdExec.cc,v 1.122 2006/05/24 07:30:21 lav Exp $ */
+/* $Id: CmdExec.cc,v 1.125.2.1 2007/04/11 05:31:03 lav Exp $ */
 
 #include <config.h>
 #include <errno.h>
@@ -62,7 +62,9 @@ static ResDecl
    res_move_background	   ("cmd:move-background","yes", ResMgr::BoolValidate,ResMgr::NoClosure),
    res_set_term_status     ("cmd:set-term-status","no", ResMgr::BoolValidate,0),
    res_term_status         ("cmd:term-status",  "", 0, 0),
-   res_trace		   ("cmd:trace",  "no",	ResMgr::BoolValidate,ResMgr::NoClosure);
+   res_trace		   ("cmd:trace",  "no",	ResMgr::BoolValidate,ResMgr::NoClosure),
+   res_parallel		   ("cmd:parallel",	   "1",  ResMgr::UNumberValidate,0),
+   res_queue_parallel	   ("cmd:queue-parallel",  "1",	 ResMgr::UNumberValidate,0);
 
 CmdExec	 *CmdExec::cwd_owner;
 CmdExec	 *CmdExec::chain;
@@ -329,7 +331,7 @@ void CmdExec::RemoveFeeder()
       queue_feeder=0;
    delete feeder;
    feeder=tmp;
-
+   Reconfig(0);
 }
 
 void CmdExec::ReuseSavedSession()
@@ -590,7 +592,8 @@ int CmdExec::Do()
       }
       if(status_line && status_line->CanShowNow())
 	 ShowRunStatus(status_line);   // this is only for top level CmdExec.
-      return m;
+      if(m != STALL || interactive || !feeder || waiting_num >= max_waiting)
+	 return m;
    }
 
    if(!interactive)
@@ -644,6 +647,8 @@ try_get_cmd:
 	       FeedCmd("exit;");
 	       return MOVED;
 	    }
+	    if(waiting_num > 0)
+	      return m;
 	    RemoveFeeder();
 	    m=MOVED;
 	    goto try_get_cmd;
@@ -824,6 +829,7 @@ CmdExec::CmdExec(FileAccess *f,LocalDirectory *c) : SessionJob(f?f:new DummyProt
    redirections=0;
 
    queue_feeder=0;
+   max_waiting=1;
 
    saved_session=0;
 
@@ -885,14 +891,8 @@ char *CmdExec::FormatPrompt(const char *scan)
    if(p && p>cwdb)
       cwdb=p+1;
 
-   char StartIgn[3], EndIgn[3];
-   /* bash adds the extra \001 too, don't know why */
-   StartIgn[0] = '\001';
-   StartIgn[1] = RL_PROMPT_START_IGNORE;
-   StartIgn[2] = 0;
-   EndIgn[0] = '\001';
-   EndIgn[1] = RL_PROMPT_END_IGNORE;
-   EndIgn[2] = 0;
+   static const char StartIgn[]={RL_PROMPT_START_IGNORE,0};
+   static const char EndIgn[]={RL_PROMPT_END_IGNORE,0};
 
    subst_t subst[] = {
       { 'a', "\007" },
@@ -955,6 +955,8 @@ void CmdExec::Reconfig(const char *name)
    // only allow explicit setting of cmd:interactive to change interactiveness.
    if(top_level && name && !strcmp(name,"cmd:interactive"))
       SetInteractive(res_interactive.QueryBool(0));
+   ResType *r=queue_feeder?&res_queue_parallel:&res_parallel;
+   max_waiting=r->Query(c);
 }
 
 void CmdExec::pre_stdout()
@@ -1311,19 +1313,21 @@ void CmdExec::FeedArgV(const ArgV *args,int start)
    xfree(cmd);
 }
 
-bool CmdExec::SameQueueParameters(CmdExec *scan)
+bool CmdExec::SameQueueParameters(CmdExec *scan,const char *this_url)
 {
-   return !strcmp(this->session->GetConnectURL(FA::NO_PATH),
-	          scan->session->GetConnectURL(FA::NO_PATH))
+   return !strcmp(this_url,scan->session->GetConnectURL(FA::NO_PATH))
       && !xstrcmp(this->slot,scan->slot);
 }
 
 /* return the CmdExec containing a queue feeder; create if necessary */
 CmdExec  *CmdExec::GetQueue(bool create)
 {
+   const char *this_url=session->GetConnectURL(FA::NO_PATH);
+   // future GetConnectURL overwrite the static buffer, save it.
+   this_url=alloca_strdup(this_url);
    for(CmdExec *scan=chain; scan; scan=scan->next)
    {
-      if(scan->queue_feeder && SameQueueParameters(scan))
+      if(scan->queue_feeder && SameQueueParameters(scan,this_url))
 	 return scan;
    }
    if(!create)
@@ -1338,6 +1342,7 @@ CmdExec  *CmdExec::GetQueue(bool create)
    queue->cmdline=xasprintf("queue (%s%s%s)",url,slot?"; ":"",slot?slot:"");
    queue->queue_feeder=new QueueFeeder(session->GetCwd(), cwd->GetName());
    queue->SetCmdFeeder(queue->queue_feeder);
+   queue->Reconfig(0);
 
    return queue;
 }
