@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ColumnOutput.cc,v 1.10 2002/07/10 08:48:51 lav Exp $ */
+/* $Id: ColumnOutput.cc,v 1.11 2006/07/24 08:50:18 lav Exp $ */
 
 #include <config.h>
 #include <sys/types.h>
@@ -228,7 +228,7 @@ void datum::append(const char *name, const char *color)
       }
    }
 
-   curwidth += mbswidth(name, MBSW_ACCEPT_INVALID|MBSW_ACCEPT_UNPRINTABLE);
+   curwidth += mbswidth(name, 0);
 }
 
 void datum::print(OutputJob *o, bool color, int skip,
