@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ResMgr.cc,v 1.64 2006/06/30 14:25:11 lav Exp $ */
+/* $Id: ResMgr.cc,v 1.64.2.2 2007/06/15 07:16:39 lav Exp $ */
 
 #include <config.h>
 
@@ -867,18 +867,25 @@ const char *ResMgr::DirReadable(char **value)
    return FileAccessible(value,R_OK|X_OK,1);
 }
 
+#ifdef HAVE_ICONV
 CDECL_BEGIN
 #include <iconv.h>
 CDECL_END
+#endif
+
 const char *ResMgr::CharsetValidate(char **value)
 {
    if(!**value)
       return 0;
+#ifdef HAVE_ICONV
    iconv_t ic=iconv_open(*value,*value);
    if(ic==(iconv_t)-1)
       return _("this encoding is not supported");
    iconv_close(ic);
    return 0;
+#else
+   return _("this encoding is not supported");
+#endif
 }
 
 const char *ResMgr::NoClosure(char **)
