@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SFtp.cc,v 1.54 2006/05/13 18:02:21 lav Exp $ */
+/* $Id: SFtp.cc,v 1.59.2.1 2007/06/15 06:28:51 lav Exp $ */
 
 #include <config.h>
 #include "SFtp.h"
@@ -31,7 +31,6 @@
 
 #include <assert.h>
 #include <errno.h>
-#include <iconv.h>
 
 #define max_buf 0x10000
 
@@ -76,15 +75,11 @@ int SFtp::Do()
    int s;
 
    // check if idle time exceeded
-   if(mode==CLOSED && send_buf && idle>0)
+   if(mode==CLOSED && send_buf && idle_timer.Stopped())
    {
-      if(now-idle_start>=idle)
-      {
-	 DebugPrint("---- ",_("Closing idle connection"),1);
-	 Disconnect();
-	 return m;
-      }
-      TimeoutS(idle_start+idle-time_t(now));
+      DebugPrint("---- ",_("Closing idle connection"),1);
+      Disconnect();
+      return m;
    }
 
    if(Error())
@@ -104,13 +99,13 @@ int SFtp::Do()
       return m;
 
    if(send_buf)
-      BumpEventTime(send_buf->EventTime());
+      timeout_timer.Reset(send_buf->EventTime());
    if(recv_buf)
-      BumpEventTime(recv_buf->EventTime());
+      timeout_timer.Reset(recv_buf->EventTime());
    if(pty_send_buf)
-      BumpEventTime(pty_send_buf->EventTime());
+      timeout_timer.Reset(pty_send_buf->EventTime());
    if(pty_recv_buf)
-      BumpEventTime(pty_recv_buf->EventTime());
+      timeout_timer.Reset(pty_recv_buf->EventTime());
 
    // check for timeout only if there should be connection activity.
    if(state!=DISCONNECTED && state!=CONNECTED
@@ -193,7 +188,7 @@ int SFtp::Do()
       ssh=new PtyShell(cmd);
       ssh->UsePipes();
       state=CONNECTING;
-      event_time=now;
+      timeout_timer.Reset();
       m=MOVED;
    }
    case CONNECTING:
@@ -314,7 +309,7 @@ void SFtp::MoveConnectionHere(SFtp *o)
    if(expect_chain_end==&o->expect_chain)
       expect_chain_end=&expect_chain;
    o->expect_chain_end=&o->expect_chain;
-   BumpEventTime(o->event_time);
+   timeout_timer.Reset(o->timeout_timer);
    ssh_id=o->ssh_id;
    state=CONNECTED;
    o->Disconnect();
@@ -820,7 +815,7 @@ void SFtp::HandleExpect(Expect *e)
 	    PropagateHomeAuto();
 	    if(!home)
 	       set_home(home_auto);
-	    LsCache::SetDirectory(this, home, true);
+	    cache->SetDirectory(this, home, true);
 	 }
       }
       break;
@@ -830,7 +825,7 @@ void SFtp::HandleExpect(Expect *e)
 	 const FileAttrs *a=((Reply_ATTRS*)reply)->GetAttrs();
 	 if(a->type!=SSH_FILEXFER_TYPE_DIRECTORY)
 	 {
-	    LsCache::SetDirectory(this,cwd,false);
+	    cache->SetDirectory(this,cwd,false);
 	    SetError(NO_FILE,strerror(ENOTDIR));
 	    break;
 	 }
@@ -838,7 +833,7 @@ void SFtp::HandleExpect(Expect *e)
 	 {
 	    cwd.Set(file);
 	    eof=true;
-	    LsCache::SetDirectory(this,cwd,true);
+	    cache->SetDirectory(this,cwd,true);
 	 }
       }
       else
@@ -1341,7 +1336,7 @@ const char *SFtp::CurrentStatus()
    return "";
 }
 
-bool SFtp::SameSiteAs(FileAccess *fa)
+bool SFtp::SameSiteAs(const FileAccess *fa) const
 {
    if(!SameProtoAs(fa))
       return false;
@@ -1350,7 +1345,7 @@ bool SFtp::SameSiteAs(FileAccess *fa)
    && !xstrcmp(user,o->user) && !xstrcmp(pass,o->pass));
 }
 
-bool SFtp::SameLocationAs(FileAccess *fa)
+bool SFtp::SameLocationAs(const FileAccess *fa) const
 {
    if(!SameSiteAs(fa))
       return false;
@@ -1397,7 +1392,7 @@ void SFtp::Reconfig(const char *name)
    if(!xstrcmp(name,"sftp:charset") && protocol_version && protocol_version<4)
    {
       if(!IsSuspended())
-	 LsCache::TreeChanged(this,"/");
+	 cache->TreeChanged(this,"/");
       const char *charset=ResMgr::Query("fish:charset",hostname);
       if(charset && *charset)
       {
@@ -2045,8 +2040,9 @@ int SFtpDirList::Do()
       const char *cache_buffer=0;
       int cache_buffer_size=0;
       int err;
-      if(use_cache && LsCache::Find(session,dir,FA::LONG_LIST,&err,
-				    &cache_buffer,&cache_buffer_size,&fset))
+      const FileSet *fset_c;
+      if(use_cache && FileAccess::cache->Find(session,dir,FA::LONG_LIST,&err,
+				    &cache_buffer,&cache_buffer_size,&fset_c))
       {
 	 if(err)
 	 {
@@ -2056,14 +2052,14 @@ int SFtpDirList::Do()
 	 ubuf=new IOBuffer(IOBuffer::GET);
 	 ubuf->Put(cache_buffer,cache_buffer_size);
 	 ubuf->PutEOF();
-	 fset=new FileSet(fset);
+	 fset=new FileSet(fset_c);
       }
       else
       {
 	 session->Open(dir,FA::LONG_LIST);
 	 ubuf=new IOBufferFileAccess(session);
-	 if(LsCache::IsEnabled())
-	    ubuf->Save(LsCache::SizeLimit());
+	 if(FileAccess::cache->IsEnabled(session->GetHostName()))
+	    ubuf->Save(FileAccess::cache->SizeLimit());
       }
    }
 
@@ -2074,7 +2070,7 @@ int SFtpDirList::Do()
    {
       if(!fset && session->IsOpen())
 	 fset=((SFtp*)session)->GetFileSet();
-      LsCache::Add(session,dir,FA::LONG_LIST,FA::OK,ubuf,fset);
+      FileAccess::cache->Add(session,dir,FA::LONG_LIST,FA::OK,ubuf,fset);
       if(use_file_set)
       {
 	 fset->Sort(fset->BYNAME,false);
@@ -2188,8 +2184,9 @@ int SFtpListInfo::Do()
       const char *cache_buffer=0;
       int cache_buffer_size=0;
       int err;
-      if(use_cache && LsCache::Find(session,"",FA::LONG_LIST,&err,
-				    &cache_buffer,&cache_buffer_size,&result))
+      const FileSet *fset_c;
+      if(use_cache && FileAccess::cache->Find(session,"",FA::LONG_LIST,&err,
+				    &cache_buffer,&cache_buffer_size,&fset_c))
       {
 	 if(err)
 	 {
@@ -2199,14 +2196,14 @@ int SFtpListInfo::Do()
 	 ubuf=new IOBuffer(IOBuffer::GET);
 	 ubuf->Put(cache_buffer,cache_buffer_size);
 	 ubuf->PutEOF();
-	 result=new FileSet(result);
+	 result=new FileSet(fset_c);
       }
       else
       {
 	 session->Open("",FA::LONG_LIST);
 	 ubuf=new IOBufferFileAccess(session);
-	 if(LsCache::IsEnabled())
-	    ubuf->Save(LsCache::SizeLimit());
+	 if(FileAccess::cache->IsEnabled(session->GetHostName()))
+	    ubuf->Save(FileAccess::cache->SizeLimit());
       }
    }
    const char *b;
@@ -2216,7 +2213,7 @@ int SFtpListInfo::Do()
    {
       if(!result && session->IsOpen())
 	 result=((SFtp*)session)->GetFileSet();
-      LsCache::Add(session,"",FA::LONG_LIST,FA::OK,ubuf,result);
+      FileAccess::cache->Add(session,"",FA::LONG_LIST,FA::OK,ubuf,result);
       result->ExcludeDots();
       result->Exclude(exclude_prefix,exclude);
       done=true;
