@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: buffer.h,v 1.40 2006/06/09 18:10:35 lav Exp $ */
+/* $Id: buffer.h,v 1.40.2.2 2007/06/15 07:16:39 lav Exp $ */
 
 #ifndef BUFFER_H
 #define BUFFER_H
@@ -30,9 +30,12 @@
 #include "xstring.h"
 
 #include <stdarg.h>
+
+#ifdef HAVE_ICONV
 CDECL_BEGIN
 #include <iconv.h>
 CDECL_END
+#endif
 
 #define GET_BUFSIZE 0x10000
 #define PUT_LL_MIN  0x2000
@@ -131,12 +134,15 @@ public:
    enum dir_t { GET, PUT };
 
 protected:
+#ifdef HAVE_ICONV
    iconv_t backend_translate;
    Buffer *untranslated;
+#endif
    dir_t mode;
    void EmbraceNewData(int len);
 
 public:
+#ifdef HAVE_ICONV
    DirectedBuffer(dir_t m) { mode=m; backend_translate=0; untranslated=0; }
    ~DirectedBuffer();
    void SetTranslation(const char *be_encoding,bool translit=true);
@@ -151,6 +157,12 @@ public:
 	    Buffer::Put(buf,size);
       }
    void Put(const char *buf) { Put(buf,strlen(buf)); }
+#else
+   DirectedBuffer(dir_t m) { mode=m; }
+   void SetTranslation(const char *be_encoding,bool translit=true) {}
+   virtual void PutTranslated(const char *buf,int size) { Put(buf,size); }
+   void ResetTranslation() {}
+#endif
    dir_t GetDirection() { return mode; }
 };
 
